 ;;;;
; ContinueLines.pb:  A PureBasic program that processes PureBasic source
; code (stored either in a file or in the system clipboard) in order to
; restore the continuity of long lines that have been split by the
; convention of using an underscore (after at least one space) at the end of
; a line as a line-continuation character.  The program removes each such
; underscore together with all immediately following consecutive whitespace
; characters including line breaks.
;
; This program takes an optional command-line argument specifying the path
; to a PureBasic source file.  If the file name contains an underscore
; immediately preceding the file-name extension (for example,
; MyProgram_.pb), the program writes the processed text to a file with that
; name minus the underscore (for example, MyProgram.pb).  Otherwise, the
; program overwrites the original file.
;
; If there is no command-line argument, the program processes and overwrites
; the text content of the system clipboard.
;;;;
EnableExplicit

IncludeFile "RegExpServices.pb"

Define .s Pattern

Define .s LineBreak
Define .l LengthOfLineBreak

Define .s SourceFilePath
Define .l LengthOfFile

Define .s InputText
Define .s OutputText
Define .s OutputFilePath

Pattern = "(?<= )_[\r\n]+\s*"

If #PB_Compiler_OS = #PB_OS_WINDOWS
    LineBreak = Chr(13) + Chr(10)
    LengthOfLineBreak = 2
Else
    LineBreak = Chr(10)
    LengthOfLineBreak = 1
EndIf

SourceFilePath = ProgramParameter()

If Len(SourceFilePath)
    If ReadFile(0, SourceFilePath)
        LengthOfFile = Lof(0)
        InputText = ""

        While Eof(0) = 0
            InputText + ReadString(0)
            InputText + LineBreak
        Wend

        CloseFile(0)

        If Len(InputText) > LengthOfFile
            InputText = RSet(InputText, Len(InputText) - LengthOfLineBreak)
        EndIf

        OutputText = RegExpSub(Pattern, InputText, "")
    Else
        MessageRequester("ContinueLines", "ERROR:  Couldn't open " + Chr(34) + SourceFilePath + Chr(34) + " for reading!  Quitting now.", #PB_MessageRequester_Ok)
        End
    EndIf

    OutputFilePath =  RegExpSub("_(?=\.[^\.]+\z)", SourceFilePath, "")

    If CreateFile(0, OutputFilePath)
        WriteString(0, OutputText)
        CloseFile(0)
    Else
        MessageRequester("ContinueLines", "ERROR:  Couldn't open " + Chr(34) + OutputFilePath + Chr(34) + " for writing!  Quitting now.", #PB_MessageRequester_Ok)
        End
    EndIf
Else
    InputText = GetClipboardText()
    OutputText = RegExpSub(Pattern, InputText, "")
    SetClipboardText(OutputText)
EndIf

MessageRequester("ContinueLines", "ContinueLines is finished!")
End 